Package: DepecheR
Version: 1.28.0
Date: 2024-08-22
Title: Determination of essential phenotypic elements of clusters in
        high-dimensional entities
biocViews:
        Software,CellBasedAssays,Transcription,DifferentialExpression,
        DataRepresentation,ImmunoOncology,Transcriptomics,Classification,Clustering,
        DimensionReduction,FeatureExtraction,FlowCytometry,RNASeq,SingleCell,
        Visualization
Authors@R: c(
    person('Jakob', 'Theorell', email = "jakob.theorell@ki.se",
        role = c("aut", "cre"), comment=c(ORCID="0000-0001-8752-3151")), 
    person('Axel', 'Theorell', email = "axel.theorell@gmail.com", 
        role = c("aut")))
Description: The purpose of this package is to identify traits in a dataset that
    can separate groups. This is done on two levels. First, clustering is 
    performed, using an implementation of sparse K-means. Secondly, the 
    generated clusters are used to predict outcomes of groups of individuals 
    based on their distribution of observations in the different clusters. As 
    certain clusters with separating information will be identified, and these 
    clusters are defined by a sparse number of variables, this method can reduce
    the complexity of data, to only emphasize the data that actually matters.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: false
RoxygenNote: 7.3.1
Depends: R (>= 4.0)
Imports: ggplot2 (>= 3.1.0), MASS (>= 7.3.51), Rcpp (>= 1.0.0), dplyr
        (>= 0.7.8), gplots (>= 3.0.1), viridis (>= 0.5.1), foreach (>=
        1.4.4), doSNOW (>= 1.0.16), matrixStats (>= 0.54.0), mixOmics
        (>= 6.6.1), moments (>= 0.14), grDevices (>= 3.5.2), graphics
        (>= 3.5.2), stats (>= 3.5.2), utils (>= 3.5), methods (>= 3.5),
        parallel (>= 3.5.2), reshape2 (>= 1.4.3), beanplot (>= 1.2),
        FNN (>= 1.1.3), robustbase (>= 0.93.5), gmodels (>= 2.18.1),
        collapse (>= 1.9.2), ClusterR (>= 1.3.2)
LinkingTo: Rcpp, RcppEigen
Suggests: uwot, testthat, knitr, rmarkdown, BiocStyle
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/DepecheR
git_branch: RELEASE_3_23
git_last_commit: 19670a0
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 21:41:55 UTC; biocbuild
Author: Jakob Theorell [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8752-3151>),
  Axel Theorell [aut]
Maintainer: Jakob Theorell <jakob.theorell@ki.se>
