# Generated by roxygen2: do not edit by hand

export(dColorPlot)
export(dColorVector)
export(dContours)
export(dDensityPlot)
export(dResidualPlot)
export(dScale)
export(dSplsda)
export(dViolins)
export(dWilcox)
export(depeche)
export(groupProbPlot)
export(nUniqueNeighDons)
export(neighSmooth)
importFrom(ClusterR,KMeans_rcpp)
importFrom(FNN,knnx.index)
importFrom(MASS,kde2d)
importFrom(Rcpp,evalCpp)
importFrom(beanplot,beanplot)
importFrom(collapse,fmode)
importFrom(doSNOW,registerDoSNOW)
importFrom(dplyr,sample_n)
importFrom(foreach,"%do%")
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(gmodels,fast.prcomp)
importFrom(gplots,heatmap.2)
importFrom(gplots,rich.colors)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,densCols)
importFrom(grDevices,dev.off)
importFrom(grDevices,palette)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(grDevices,rainbow)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,contour)
importFrom(graphics,hist)
importFrom(graphics,image)
importFrom(graphics,legend)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.new)
importFrom(graphics,text)
importFrom(matrixStats,rowMedians)
importFrom(methods,is)
importFrom(mixOmics,splsda)
importFrom(mixOmics,tune.splsda)
importFrom(moments,kurtosis)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(reshape2,melt)
importFrom(robustbase,colMedians)
importFrom(stats,as.dendrogram)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,median)
importFrom(stats,order.dendrogram)
importFrom(stats,p.adjust)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(utils,globalVariables)
importFrom(utils,tail)
importFrom(utils,write.csv)
importFrom(viridis,inferno)
importFrom(viridis,magma)
importFrom(viridis,plasma)
importFrom(viridis,viridis)
useDynLib(DepecheR)
