#' A 14 color flow cytometry dataset for example execution and playing around
#'
#' This dataset is a 14 color pre-compensated, transformed flow cytometry
#' dataset focusing on cytotoxic lymphocytes, where dead cells have been
#' removed. To make examples very obvious, differences have been artificially
#' exaggregated in the data. The dataset is produced by J. Theorell.
#'
#'
#' @docType data
#'
#' @usage data("testData")
#'
#' @format An object of class \code{'data.frame'};
#'
#' @keywords datasets
"testData"
