#' A depeche clustering of the testData set
#'
#' This is a depeche clustering of the testData dataset. It has
#' been generated with the Rtsne.multicore package
#'
#' produced by J. Theorell.
#'
#'
#' @docType data
#'
#' @usage data(testDataDepeche)
#'
#' @format An object of class \code{'list'};
#'
#' @keywords datasets
#'
"testDataDepeche"
