#' SNE of the testData set
#'
#' This is a t-distributed stochastic neighbor embedding of the testData
#' dataset. It has been generated with the Rtsne.multicore package.
#'
#'
#' @docType data
#'
#' @usage data(testDataSNE)
#'
#' @format An object of class \code{'list'};
#'
#' @keywords datasets
#' #'
#' @references Jesse H. Krijthe (2015). Rtsne: T-Distributed Stochastic Neighbor
#' Embedding using a Barnes-Hut Implementation
#' (\href{https://github.com/RGLab/Rtsne.multicore/issues/5}{GitHub})
#'
"testDataSNE"
